/*           
UserOptions.cc
*/

#include "UserOptions.h"

extern IMAP *theimap;

UserOptions::UserOptions (const char *abasepath, const char *auser, const char *aimapserver)
  {
  validator = 0;
  WorkingWithCookies = false;
  UseJavaScript = true;
  ConfirmPurge = DEFOPT_CONFIRMPURGE;
  ConfirmLogout = DEFOPT_CONFIRMLOGOUT;
  timeout = DEFTIMEOUT;
  InitStatusLine ();
  user = XString (auser);
  imapserver = XString (aimapserver);
  basepath = XString (abasepath);
  userdir = XString (ReturnUsersDir (abasepath, auser, aimapserver));
  fnuseroptions = XString (ReturnUserOptionsFileName(abasepath, auser, aimapserver));
  fnsignature = XString (ReturnSignatureFileName(abasepath, auser, aimapserver));
  fnsavedmsg  = XString (ReturnUserSavedMsgFileName (abasepath, auser, aimapserver));
  userattdir = XString (ReturnUserAttachsDir(abasepath, auser, aimapserver));
  TheConf = new Configuration ((char *)fnuseroptions.cstr(), false);
  signature = XString ("");
  sort_ab = 0;
  CurrentPosition = -1;
  strcpy (skin, DEFOPT_SKIN);
  setSkin (skin);
  IMAPPORT = DEFIMAPPORT;
  initStr (MAILDOMAIN);
  initStr (IMAPSERVER);
  initStr (SMTPSERVER);
  pageid = -1;
  lastcmd = -1;
  SentmailName = XString (DEFOPT_SENTMAILBOX);
  MAILBOXSIZE = -1;
  BeginPage = 1;
  LoadOptions ();
  msgtoanswer = -1;
  }
  
UserOptions::~UserOptions ()  
  {
  }  

bool UserOptions::IsWorkingWithCookies (void)
  {
  return WorkingWithCookies;
  }  

void UserOptions::setWorkingWithCookies (bool b)
  {
  WorkingWithCookies = b;
  }  

bool UserOptions::get_use_javascript (void)
  {
  return UseJavaScript;
  }

void UserOptions::set_use_javascript (bool b)
  {
  UseJavaScript = b;
  }
  
void UserOptions::leeConf ()
  {
  if (TheConf->reload() == false) {TheConf->newFile (DEF_USEROPT_FILE); LoadOptions ();}  
  }

void UserOptions::setMailboxPrefix (const char *amailboxprefix)
  {
  strncpy (MAILBOXPREFIX, amailboxprefix, CMAXBUFFER);
  }
  
char *UserOptions::getMailboxPrefix (void)
  {
  return xstrdup (MAILBOXPREFIX);
  }      

void UserOptions::setMailDomain (const char *amaildomain)
  {
  strncpy (MAILDOMAIN, amaildomain, CMAXIPNAME);
  }
  
char *UserOptions::getMailDomain (void)
  {
  return xstrdup (MAILDOMAIN);
  }      
  
void UserOptions::setImapServer (const char *aimapserver)
  {
  strncpy (IMAPSERVER, aimapserver, CMAXIPNAME);
  imapserver = XString (aimapserver);
  userdir = XString (ReturnUsersDir                  ((char *)basepath.cstr(), (char *)user.cstr(), IMAPSERVER));
  fnuseroptions = XString (ReturnUserOptionsFileName ((char *)basepath.cstr(), (char *)user.cstr(), IMAPSERVER));
  fnsignature = XString (ReturnSignatureFileName     ((char *)basepath.cstr(), (char *)user.cstr(), IMAPSERVER)); 
  fnsavedmsg  = XString (ReturnUserSavedMsgFileName  ((char *)basepath.cstr(), (char *)user.cstr(), IMAPSERVER));
  userattdir = XString (ReturnUserAttachsDir         ((char *)basepath.cstr(), (char *)user.cstr(), IMAPSERVER));
  TheConf->setFileName ((char *)fnuseroptions.cstr());
  TheConf = new Configuration ((char *)fnuseroptions.cstr(), false);
  LoadOptions ();
  }  
  
char *UserOptions::getImapServer (void)
  {
  return xstrdup (IMAPSERVER);
  }
  
void UserOptions::setImapPort (int aimapport)
  {
  IMAPPORT = aimapport;
  }
  
int UserOptions::getImapPort (void)
  {
  return IMAPPORT;
  }

void UserOptions::setSMTPServer (const char *asmtpserver)
  {
  strncpy (SMTPSERVER, asmtpserver, CMAXIPNAME);
  }
  
char *UserOptions::getSMTPServer (void)
  {
  return xstrdup (SMTPSERVER);
  }

void UserOptions::setCurrentPosition (long pos, long total)
  {
  CurrentPosition = pos;
  TotalMsgs       = total;
  if (CurrentPosition == 0) CurrentPosition = TotalMsgs;
  if (CurrentPosition > TotalMsgs) CurrentPosition = TotalMsgs;
  if (CurrentPosition < 1) CurrentPosition = 1;
  }

long UserOptions::getCurrentPosition (void)
  {
  return CurrentPosition;
  }
  
long UserOptions::getBeginPage (void)
  {
  long p;
  ldiv_t ld;
  if (getMsgsPerIndexPage() == 0)
    {
    BeginPage = 1;
    }
  else
    {
    ld = ldiv (CurrentPosition - 1, getMsgsPerIndexPage());
    p = ld.quot * getMsgsPerIndexPage();
    BeginPage = p + 1;
    }
  return BeginPage;
  }        

long UserOptions::getTotalMsgs (void)
  {
  return TotalMsgs;
  }

void UserOptions::setMailboxSize (long asize)
  {
  MAILBOXSIZE = asize;
  }
  
long UserOptions::getMailboxSize (void)    
  {
  return MAILBOXSIZE;
  }      

void UserOptions::InitStatusLine (void)
  {
  initStr (StatusLine, HTML_SP);
  }

const char *UserOptions::getUsername (void)
  {
  return user.cstr();
  }  

const char *UserOptions::getAttachDir (void)
  {
  return userattdir.cstr();
  }  

const char *UserOptions::getStatusLine (void)
  {
  return StatusLine;
  }  

void UserOptions::setNumMsgToReply (int value)
  {
  msgtoanswer = value;
  }
  
int UserOptions::getNumMsgToReply(void)
  {
  return msgtoanswer;
  } 

void UserOptions::setStatusLine (char *newstatusline, char *htmlcolor)
  {
  sprintf (StatusLine, "<FONT face=\"%s\" size=\"%d\" color=\"%s\">%s</FONT>", 
           getSkinAtribProp ("FONT_STATUS"), getSkinAtribPropI ("FONT_STATUS_SIZE"),
           htmlcolor, newstatusline);
  }
        
int UserOptions::getValidator (void)
  {
  return validator;
  }  
  
void UserOptions::IncValidator (void)
  {
  ++validator;
  }  

void UserOptions::setInsertSignatureInNewMsg (bool value)
  {
  InsertSignatureInNewMsg = value;
  }
  
bool UserOptions::getInsertSignatureInNewMsg (void)
  {
  return InsertSignatureInNewMsg;
  }  
  
void UserOptions::setMsgsPerIndexPage (long value)
  {
  if (value < 0) MsgsPerIndexPage = DEFOPT_MSGSPERINDEXPAGE;
  else MsgsPerIndexPage = value;
  }

long UserOptions::getMsgsPerIndexPage (void)
  {
  return MsgsPerIndexPage;
  }
  
void UserOptions::setPersonalName (char *value)
  {
  personalname = XString (value);
  }

char *UserOptions::getPersonalName (void)
  {
  return (char *)(personalname.cstr());
  }  
  
void UserOptions::setSaveMsgSentMail (bool value)
  {
  SaveMsgSentMail = value;
  }
  
bool UserOptions::getSaveMsgSentMail (void)
  {
  return SaveMsgSentMail;
  }  
  
void UserOptions::setReplyFrom (bool value)
  {
  ReplyFrom = value;
  }
  
bool UserOptions::getReplyFrom (void)
  {
  return ReplyFrom;
  }    
  
void UserOptions::setForwAttachs (bool value)
  {
  ForwAttachs = value;
  }
  
bool UserOptions::getForwAttachs (void)
  {
  return ForwAttachs;
  }
        
void UserOptions::setTruncateLengthReadingMsg (int value)
  {
  TruncateLengthReadingMsg = value;
  }
  
int UserOptions::getTruncateLengthReadingMsg (void)
  {
  return TruncateLengthReadingMsg;
  }    
  
void UserOptions::setWidthWriteArea (int value)
  {
  WidthWriteArea = value;
  }
  
int UserOptions::getWidthWriteArea (void)
  {
  return WidthWriteArea;
  }  
  
void UserOptions::setHeightWriteArea (int value)
  {
  HeightWriteArea = value;
  }
  
int UserOptions::getHeightWriteArea (void)
  {
  return HeightWriteArea;
  }    
  
char *UserOptions::getSignature (void)
  {
  return (char *)(signature.cstr());  
  }

int UserOptions::getRefreshTime (void)
  {
  return RefreshTime;
  }    
  
void UserOptions::setRefreshTime (int value)
  {
  RefreshTime = value;
  }

bool UserOptions::SaveOptions (char *tmpfilename)
  {
  FILE *fs;
  StringList *SLSign;
  Configuration *XConf;
  XConf = new Configuration (tmpfilename, false);
  XConf->reload();
  SLSign = new StringList ();
  TheConf->setInt    ("main", "msgsperindexpage",XConf->getInt("main", "msgsperindexpage", DEFOPT_MSGSPERINDEXPAGE));
  TheConf->setInt    ("main", "insigninnewmsg",  XConf->getInt("main", "insigninnewmsg", DEFOPT_INSERTSIGNATUREINNEWMSG));  
  TheConf->setString ("main", "personalname",    XConf->getString("main", "personalname", "", 100), 100);  
  TheConf->setInt    ("main", "savemsgsentmail", XConf->getInt("main", "savemsgsentmail", DEFOPT_SAVEMSGSENTMAIL));
  TheConf->setInt    ("main", "replyfrom",       XConf->getInt("main", "replyfrom", DEFOPT_REPLYFROM));
  TheConf->setInt    ("main", "forwattachs",     XConf->getInt("main", "forwattachs", DEFOPT_FORW_ATTACHS));
  TheConf->setInt    ("main", "heightwritearea", XConf->getInt("main", "heightwritearea", DEFOPT_HEIGHTWRITEAREA));
  TheConf->setInt    ("main", "widthwritearea",  XConf->getInt("main", "widthwritearea", DEFOPT_WIDTHWRITEAREA));   
  TheConf->setInt    ("main", "truncatelengthreadingmsg",  XConf->getInt("main", "truncatelengthreadingmsg", DEFOPT_TRUNCATELENGTHREADINGMSG));        
  TheConf->setString ("main", "skin",            XConf->getString("main", "skin", "DEFAULT", 100), 100);  
  TheConf->setInt    ("main", "refreshtime",     XConf->getInt("main", "refreshtime", DEFOPT_REFRESHTIME)); 
  TheConf->setString ("main", "sentmailname",    XConf->getString("main", "sentmailname", DEFOPT_SENTMAILBOX, 50), 50);      
  TheConf->setInt    ("main", "confirmpurge",    XConf->getInt("main", "confirmpurge",  DEFOPT_CONFIRMPURGE));  
  TheConf->setInt    ("main", "confirmlogout",   XConf->getInt("main", "confirmlogout", DEFOPT_CONFIRMLOGOUT));
  TheConf->ConfSave();
  XConf->getSection ("signature", SLSign);
  fs = fopen ((char *)(fnsignature.cstr()), "w"); if (fs == NULL) return false;
  for (int yy=0; yy < SLSign->Count(); ++yy)
    {
    fprintf (fs, "%s\n", SLSign->getString(yy).cstr());  
    }
  fclose (fs);
  delete SLSign;
  delete XConf;
  DeleteFile (tmpfilename);
  LoadOptions ();
  return true;
  }  

void UserOptions::LoadOptions (void)
  {
  if (FileExists ((char *)fnuseroptions.cstr()) == false)
    {
    setMsgsPerIndexPage (DEFOPT_MSGSPERINDEXPAGE);
    setInsertSignatureInNewMsg (DEFOPT_INSERTSIGNATUREINNEWMSG);
    setPersonalName ("");
    setSaveMsgSentMail (DEFOPT_SAVEMSGSENTMAIL);
    setReplyFrom  (DEFOPT_REPLYFROM);
    setForwAttachs (DEFOPT_FORW_ATTACHS);
    setWidthWriteArea (DEFOPT_WIDTHWRITEAREA);
    setHeightWriteArea (DEFOPT_HEIGHTWRITEAREA);
    setTruncateLengthReadingMsg (DEFOPT_TRUNCATELENGTHREADINGMSG);
    setSkin (DEFOPT_SKIN);
    setRefreshTime (DEFOPT_REFRESHTIME);
    setSentMailName (DEFOPT_SENTMAILBOX);
    setConfirmPurge (DEFOPT_CONFIRMPURGE);
    setConfirmLogout (DEFOPT_CONFIRMLOGOUT);
    }
  else
    {
    int nummmsgsperpage, heightwritearea, widthwritearea, truncate;
    TheConf->reload ();
    nummmsgsperpage = TheConf->getInt ("main", "msgsperindexpage", DEFOPT_MSGSPERINDEXPAGE); if (nummmsgsperpage < 0) nummmsgsperpage = DEFOPT_MSGSPERINDEXPAGE; setMsgsPerIndexPage(nummmsgsperpage);
    setInsertSignatureInNewMsg (TheConf->getInt ("main", "insigninnewmsg", DEFOPT_INSERTSIGNATUREINNEWMSG));
    setPersonalName (TheConf->getString("main", "personalname", "", 100));
    setSaveMsgSentMail(TheConf->getInt("main", "savemsgsentmail", DEFOPT_SAVEMSGSENTMAIL));
    setReplyFrom (TheConf->getInt("main", "replyfrom", DEFOPT_REPLYFROM));
    setForwAttachs (TheConf->getInt("main", "forwattachs", DEFOPT_FORW_ATTACHS));
    heightwritearea = TheConf->getInt("main", "heightwritearea", DEFOPT_HEIGHTWRITEAREA); if (heightwritearea < 2) heightwritearea = DEFOPT_HEIGHTWRITEAREA; setHeightWriteArea(heightwritearea);
    widthwritearea = TheConf->getInt("main", "widthwritearea", DEFOPT_WIDTHWRITEAREA); if (widthwritearea < 2) widthwritearea = DEFOPT_WIDTHWRITEAREA; setWidthWriteArea(widthwritearea);
    truncate = TheConf->getInt("main", "truncatelengthreadingmsg", DEFOPT_TRUNCATELENGTHREADINGMSG); if (truncate < 10) truncate = DEFOPT_WIDTHWRITEAREA; setTruncateLengthReadingMsg(truncate);    
    setSkin (TheConf->getString("main", "skin", "DEFAULT", MAXLENGTHSKIN));
    RefreshTime = TheConf->getInt("main", "refreshtime", DEFOPT_REFRESHTIME); if ((RefreshTime < MIN_TIME_TO_REFRESH) && (RefreshTime != 0)) RefreshTime = MIN_TIME_TO_REFRESH; 
    setSentMailName (TheConf->getString("main", "sentmailname", DEFOPT_SENTMAILBOX, 50));
    setConfirmPurge (TheConf->getInt("main", "confirmpurge", DEFOPT_CONFIRMPURGE));
    setConfirmLogout (TheConf->getInt("main", "confirmlogout", DEFOPT_CONFIRMLOGOUT));
    }
  if (FileExists ((char *)(fnsignature.cstr())) == true) 
    {
    TBuffer abuf;
    FILE *fr;
    fr = fopen ((char *)fnsignature.cstr(), "r");
    signature = "";
    while (fgets (abuf, CMAXBUFFER, fr) != NULL)
      {
      signature += abuf;
      }
    fclose (fr);
    }
  else 
    {
    signature =  XString("--\n********************************\n");  
    signature +=          user;
    signature +=           "\n********************************\n";
    }
  }  

int UserOptions::numAttachs (void)
  {
  return SLAttachList.Count ();
  }
 
char *UserOptions::getAttachInfo (int p)
  {
  if (p >= SLAttachListInfo.Count ()) return "";
  return ((char *)SLAttachListInfo.getString (p).cstr());
  }
 
char *UserOptions::getFileNameAttach (int p)
  {
  if (p >= SLAttachList.Count ()) return "";
  return ((char *)SLAttachList.getString (p).cstr());
  }      

char *UserOptions::getTrueFileNameAttach (int p)
  {
  if (p >= SLAttachList.Count ()) return "";
  XString XS;
  XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr(), imapserver.cstr())); XS += "/"; XS += getFileNameAttach(p);
  return xstrdup((char *)XS.cstr());
  }

int UserOptions::addFileNameAtach (char *newfile, char *info)
  {
  if (numAttachs () >= MAXNUMATTACHSINSEND) {return 0;}
  if (IsEmpty(newfile)) {return 0;}
  XString XS;
  XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr(), imapserver.cstr())); XS += "/"; XS += newfile;  
  if (!FileExists (XS.cstr())) {return 0;}
  if (FileSize (XS.cstr()) <= 0) {return 0;}
  if (!SLAttachList.contains(newfile)) 
    {
    SLAttachList.Add (newfile);
    //2 IMAGE/GIF 154387 bytes, "un comentario", "logo.gif"
    SLAttachListInfo.Add (info);
    return 1;
    }
  else return 0;  
  }

//SUPONGO QUE LA LISTA A BORRAR VIENE ORDENADA
// p.e. 1,5,8,40
int UserOptions::delFileNameAtach (char *attachnumbers)
  {
  if (attachnumbers == NULL) return 0;
  TBuffer abuf;
  char c, letra[2];
  int res, totaltodel;
  StringList SLtoDel;
  letra[1] = '\0';
  abuf[0]  = '\0';
  SLtoDel.Clear();
  for (int i=0; i < (xsint)strlen(attachnumbers); ++i)
    {
    c = attachnumbers[i];
    if (c != ',')
      {
      letra[0] = c;
      strcat (abuf, letra);
      }
    else
      {
      res = xatoidef (abuf, -1);
      if (res > -1) SLtoDel.Add(abuf);
      abuf[0] = '\0';
      }
    }
  if ((xsint)strlen(abuf) > 0)
    {
    res = xatoidef (abuf, -1);
    if (res > -1) SLtoDel.Add(abuf);
    }  
  totaltodel = SLtoDel.Count();  
  for (int k = SLtoDel.Count() - 1;  k >= 0; --k)
    {
    int R;
    R = atoi(SLtoDel.getString(k).cstr());
    XString XS;
    XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr(), imapserver.cstr())); 
    XS += "/"; XS += SLAttachList.getString(R).cstr();
    DeleteFile (XS.cstr());
    SLAttachList.removeElementAt (R);
    SLAttachListInfo.removeElementAt (R);
    } 
  return totaltodel;
  }
  
void UserOptions::delAllAttachs (void)
  {
  for (int i = 0; i < SLAttachList.Count(); ++i)
    {
    XString XS;
    XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr(), imapserver.cstr())); 
    XS += "/"; XS += SLAttachList.getString(i).cstr();
    DeleteFile (XS.cstr());
    }     
  SLAttachList.Clear();  
  SLAttachListInfo.Clear();
  }  

void UserOptions::setSort_ab (int value)
  {
  sort_ab = value;
  }
  
int UserOptions::getSort_ab (void)
  {
  return sort_ab;
  }                  
  
char *UserOptions::getfnSavedMsg (void)
  {
  return ((char *)fnsavedmsg.cstr());
  }
  
void UserOptions::setLastCmd (int value)
  {
  lastcmd = value;
  }
  
int UserOptions::getLastCmd (void)
  {
  return lastcmd;
  }                    

/*Vuelca por pantalla la pila*/
void UserOptions::vuelcaPila (void)
  {
  printf ("***************STACK SIZE=%d*******************\n", PilaCmds.Count());
  for (int j = PilaCmds.Count () - 1; j >= 0; --j)
    {
    printf ("---%d %s---\n", j, PilaCmds.getString(j).cstr());
    }
  }

/*Ajusto como pageid la que me llega por el cgi*/  
void UserOptions::setPageId (int value)
  {
  pageid = value;
  if ((pageid < PilaCmds.Count ()) && (pageid > -1) && (PilaCmds.Count () > 1))
    {
    for (int j = PilaCmds.Count () - 1; j >= pageid; --j)
      {
      PilaCmds.removeElementAt(j);
      }
    }
  }

/*Devuelvo una nueva pageid para la pagina nueva*/
int UserOptions::getPageId (void)
  {
  return pageid + 1;
  }    

void UserOptions::addLastStringCmd (char *stcmd)
  {                                         
  PilaCmds.Add(stcmd); 
  //vuelcaPila();      
  }

/*Devuelve el comando back correspondiente a este pageid*/  
/*La pila va desde 0 a PilaCmds.Count () - 1*/
char *UserOptions::getLastStringCmd (int cmdactual, char *lang, char *connid, bool withhtml)
  {
  TBuffer defbuf;
  TSBuffer beghtml, endhtml, linea, aconnid, stlang, ipclient, stcmd, parm1, parm2, parm3, parm4, allparms;
  int pagprevia, cmd, otropageid, cmdcomodin, cmdcomodin2, avalidator, alastcmd;
  
  if (withhtml == true) {initStr (beghtml, "<A HREF=\""); initStr (endhtml, "\">");}
  else {initStr (beghtml); initStr (endhtml);}  
  
  sprintf (defbuf, "%s%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/%s", 
           beghtml, PATH_POSTMAN, 
           lang, (char *)user.cstr(), connid, getValidator(), pageid, cmdactual, 
           theimap->getPosListFromNumMsg(getCurrentPosition()) + 1, 
           endhtml);
  pagprevia = pageid;
  cmd = CMD_MB_INDEX;
  cmdcomodin  = CMD_AB_ADDFIELDS; //Uno cualquiera fijo
  cmdcomodin2 = CMD_AB_ADDFIELDS; //idem  
  do
    {
    --pagprevia;
    if (pagprevia >= 0)
      {
      strncpy (linea, PilaCmds.getString(pagprevia).cstr(), CSMALLBUFFER);
      sscanf (linea, "%s %d %d %d %s %s %s %s %s %s %s", aconnid, &otropageid, &avalidator, &alastcmd, stlang, ipclient, stcmd, parm1, parm2, parm3, parm4);
      sscanf (linea, "%s %d %d %d %s %s %s %[^]]s",      aconnid, &otropageid, &avalidator, &alastcmd, stlang, ipclient, stcmd, allparms);
      cmd = TranslateCmd (stcmd);
      cmdcomodin  = CMD_AB_ADDFIELDS; //Uno cualquiera fijo
      cmdcomodin2 = CMD_AB_ADDFIELDS; //idem
      if (cmdactual == CMD_MG_DISPLAY) {cmdcomodin = CMD_MG_DISPLAY;}
      else if (cmdactual == CMD_MG_DELDISPLAY) {cmdcomodin = CMD_MG_DELDISPLAY;}
      else if (cmdactual == CMD_AB_ADDFIELDS){cmdcomodin = CMD_AB_SHOWALL; cmdcomodin2 = CMD_AB_FROM_CM;}      
      else if (cmdactual == CMD_AB_SHOWALL)  {cmdcomodin = CMD_AB_SHOWALL; cmdcomodin2 = CMD_AB_FROM_CM;}      
      else if (cmdactual == CMD_CM_SAVE)     {cmdcomodin = CMD_CM;}
      else if (cmdactual == CMD_CM_CANCEL)   {cmdcomodin = CMD_CM;}      
      else if (cmdactual == CMD_CM_CLEANALL) {cmdcomodin = CMD_CM; cmdcomodin2 = CMD_CM_FORWARD;}      
      }                                   
    }
  while (((cmd == CMD_AB_ADDFIELDS) || 
          (cmd == CMD_AB_SAVEENTRY) || (cmd == CMD_AB_NEWENTRY) || (cmd == CMD_AB_TAKEADDRESS) || 
          (cmd == CMD_AB_EDITENTRY) ||
          (cmd == CMD_CM_CANCEL)    || (cmd == CMD_CM_CLEANALL) || (cmd == CMD_CM_SAVE)        || 
          (cmd == CMD_CONF_ACT)     ||
          (cmd == CMD_MB_EXPUNGE)   || 
          (cmd == CMD_AT_ERASE)     || (cmd == CMD_AT_SHOW)     || (cmd == CMD_AT_SAVE)        ||
          (cmd == CMD_MG_DELDISPLAY)||
          (cmd == cmdcomodin)       || (cmd == cmdcomodin2) ||
          (cmd == cmdactual)) 
        && (pagprevia >= 0));
  if (pagprevia < 0) pagprevia = 0;       
  switch (cmd)
    {
    case CMD_AB_FROM_CM:
      {
      sprintf (defbuf, "%s%s/ab/ab_from_cm/%s/%s/%s/%d/%d/%d/%ld/%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);
      break;
      }           
    case CMD_AB_SHOWALL:
      {
      sprintf (defbuf, "%s%s/ab/ab_showall/%s/%s/%s/%d/%d/%d/%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, endhtml);
      break;
      }       
    case CMD_CM:
      {
      sprintf (defbuf, "%s%s/cm/noop/%s/%s/%s/%d/%d/%d/%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, endhtml);
      break;
      }   
    case CMD_CM_FORWARD:
      {
      sprintf (defbuf, "%s%s/cm_forward/noop/%s/%s/%s/%d/%d/%d/%ld/%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);
      break;      
      }           
    case CMD_CM_REPLY:
      {
      sprintf (defbuf, "%s%s/cm_reply/noop/%s/%s/%s/%d/%d/%d/%ld/%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);
      break;      
      }       
    case CMD_MG_DISPLAY: case CMD_MG_DELDISPLAY:   
      {
      sprintf (defbuf, "%s%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);      
      break;
      }
    case CMD_AB: case CMD_MB_INDEX: case CMD_OP: case CMD_MB_SHOW:  
    default:
      {
      break;
      }  
    }
         
  return (xstrdup(defbuf));                
  }  

int UserOptions::getSkinAtribPropI (char *propiedad)
  {
  if      (strcmp (propiedad, "FONT_TABLE_SIZE") == 0)        {return FONT_TABLE_SIZE;}
  else if (strcmp (propiedad, "FONT_MAIN_SIZE") == 0)         {return FONT_MAIN_SIZE;}  
  else if (strcmp (propiedad, "FONT_STATUS_SIZE") == 0)       {return FONT_STATUS_SIZE;}    
  else if (strcmp (propiedad, "FONT_BOTONERA_SIZE") == 0)     {return FONT_BOTONERA_SIZE;}      
  else return -1;
  }  
  
char *UserOptions::getSkinAtribProp (char *propiedad)
  {
  if      (strcmp (propiedad, "COLOR_BGTABLE") == 0)          {return COLOR_BGTABLE;}
  else if (strcmp (propiedad, "COLOR_BGTABLECOPYRIGHT") == 0) {return COLOR_BGTABLECOPYRIGHT;}  
  else if (strcmp (propiedad, "COLOR_BGBOTONERA") == 0)       {return COLOR_BGBOTONERA;}  
  else if (strcmp (propiedad, "COLOR_BGTABLEMSGINDEX") == 0)  {return COLOR_BGTABLEMSGINDEX;}    
  else if (strcmp (propiedad, "COLOR_BGBODY") == 0)           {return COLOR_BGBODY;}      
  else if (strcmp (propiedad, "COLOR_BGTEXTO") == 0)          {return COLOR_BGTEXTO;}      
  else if (strcmp (propiedad, "COLOR_BGFILASMSGINDEX") == 0)  {return COLOR_BGFILASMSGINDEX;}      
  else if (strcmp (propiedad, "COLOR_BGFILASADDBOOK") == 0)   {return COLOR_BGFILASADDBOOK;}      
  else if (strcmp (propiedad, "COLOR_BGFILASREADINGMSG") == 0){return COLOR_BGFILASREADINGMSG;}      
  else if (strcmp (propiedad, "COLOR_FONT_TABLE") == 0)       {return COLOR_FONT_TABLE;}      
  else if (strcmp (propiedad, "COLOR_FONT_MAIN") == 0)        {return COLOR_FONT_MAIN;}      
  else if (strcmp (propiedad, "COLOR_FONT_STATUS") == 0)      {return COLOR_FONT_STATUS;}   
  else if (strcmp (propiedad, "COLOR_FONT_STATUSLINE") == 0)  {return COLOR_FONT_STATUSLINE;}      
  else if (strcmp (propiedad, "FONT_TABLE") == 0)             {return FONT_TABLE;}
  else if (strcmp (propiedad, "FONT_MAIN") == 0)              {return FONT_MAIN;}        
  else if (strcmp (propiedad, "FONT_STATUS") == 0)            {return FONT_STATUS;}        
  else if (strcmp (propiedad, "IMG_BG") == 0)                 {return IMG_BG;}        
  else return "";
  }
 
char *UserOptions::getSkin (void)
  {
  return xstrdup(skin);
  }  
 
void UserOptions::setSkin (char *askin)
  {  
  xucase (askin);
  strcpy (skin, askin);
  if (strcmp (askin, "OLIVE") == 0) 
    {
    strncpy (COLOR_BGTABLE,              "#888800", MAXLENGTHCOLORHTML); 
    strncpy (COLOR_BGTABLECOPYRIGHT,     "#888866", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGBOTONERA,           "#888866", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGTABLEMSGINDEX,      "#888800", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_BGBODY,               "#AAAA66", MAXLENGTHCOLORHTML);        
    strncpy (COLOR_BGTEXTO,              "#000000", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASMSGINDEX,      "#888866", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASADDBOOK,       "#FFF0DC", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_TABLE,           "#222200", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_MAIN,            "#222200", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_STATUS,          "#AAFF00", MAXLENGTHCOLORHTML);
    strncpy (COLOR_FONT_STATUSLINE,      "#111111", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASREADINGMSG,    "#888866", MAXLENGTHCOLORHTML);
    strncpy (FONT_TABLE,                 "Arial, Helvetica, sans-serif", MAXLENGTHFONTNAME);    
    strncpy (FONT_MAIN,                  "Times New Roman", MAXLENGTHFONTNAME);        
    strncpy (FONT_STATUS,                "Lucida Blackletter", MAXLENGTHFONTNAME);            
    FONT_TABLE_SIZE  = 3;
    FONT_MAIN_SIZE   = 3;
    FONT_STATUS_SIZE = 3;
    FONT_BOTONERA_SIZE = 1;
    //sprintf (IMG_BG, "%s/background_pink.jpg", PATH_IMAGENS);
    sprintf (IMG_BG, " ");
    }                          
  else if (strcmp (askin, "RED") == 0) 
    {
    strncpy (COLOR_BGTABLE,              "#DD5555", MAXLENGTHCOLORHTML); 
    strncpy (COLOR_BGTABLECOPYRIGHT,     "#DD6666", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGBOTONERA,           "#DD6666", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGTABLEMSGINDEX,      "#DD6666", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_BGBODY,               "#DDAAAA", MAXLENGTHCOLORHTML);        
    strncpy (COLOR_BGTEXTO,              "#000000", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASMSGINDEX,      "#BBAAAA", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASADDBOOK,       "#BBAAAA", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_TABLE,           "#FFFF00", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_MAIN,            "#000000", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_STATUS,          "#000000", MAXLENGTHCOLORHTML);
    strncpy (COLOR_FONT_STATUSLINE,      "#111111", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASREADINGMSG,    "#BBAAAA", MAXLENGTHCOLORHTML);
    strncpy (FONT_TABLE,                 "Arial, Helvetica, sans-serif", MAXLENGTHFONTNAME);    
    strncpy (FONT_MAIN,                  "Times New Roman", MAXLENGTHFONTNAME);        
    strncpy (FONT_STATUS,                "Lucida Blackletter", MAXLENGTHFONTNAME);            
    FONT_TABLE_SIZE  = 3;
    FONT_MAIN_SIZE   = 3;
    FONT_STATUS_SIZE = 3;
    FONT_BOTONERA_SIZE = 1;
    //sprintf (IMG_BG, "%s/background_pink.jpg", PATH_IMAGENS);
    sprintf (IMG_BG, " ");
    }                              
  else //APARIENCIA POR DEFECTO
    {
    strncpy (COLOR_BGTABLE,             "#CCCCCC", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGTABLECOPYRIGHT,    "#CCCCCC", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGBOTONERA,          "#CCCCCC", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGTABLEMSGINDEX,     "#CCCCCC", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_BGBODY,              "#CCCCCC", MAXLENGTHCOLORHTML);        
    strncpy (COLOR_BGTEXTO,             "#000000", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASMSGINDEX,     "#E6E6E6", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASADDBOOK,      "#E6E6E6", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_TABLE,          "#000000", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_MAIN,           "#000000", MAXLENGTHCOLORHTML);    
    strncpy (COLOR_FONT_STATUS,         "#000000", MAXLENGTHCOLORHTML);
    strncpy (COLOR_FONT_STATUSLINE,     "#00AA00", MAXLENGTHCOLORHTML);
    strncpy (COLOR_BGFILASREADINGMSG,   "#FFF0DC", MAXLENGTHCOLORHTML);
    strncpy (FONT_TABLE,                "PrimaSans BT,Verdana,sans-serif", MAXLENGTHFONTNAME);    
    strncpy (FONT_MAIN,                 "PrimaSans BT,Verdana,sans-serif", MAXLENGTHFONTNAME);        
    strncpy (FONT_STATUS,               "PrimaSans BT,Verdana,sans-serif", MAXLENGTHFONTNAME);            
    FONT_TABLE_SIZE  = 2;
    FONT_MAIN_SIZE   = 2;
    FONT_STATUS_SIZE = 2;
    FONT_BOTONERA_SIZE = 1;
    sprintf (IMG_BG, "%s/background.jpg", PATH_IMAGENS);
    }
  }

char *UserOptions::getSentMailName (void)
  {
  if ((char *)SentmailName.cstr() == NULL) return DEFOPT_SENTMAILBOX;
  return xstrdup((char *)SentmailName.cstr());
  }
  
void UserOptions::setSentMailName (char *smname)
  {
  if ((smname == NULL) || (strcmp (smname, "") == 0)) return;
  SentmailName = XString (smname);
  }

bool UserOptions::getConfirmPurge (void)
  {
  return ConfirmPurge;
  }
  
void UserOptions::setConfirmPurge (bool b)
  {
  ConfirmPurge = b;
  }
  
bool UserOptions::getConfirmLogout (void)
  {
  return ConfirmLogout;  
  }
  
void UserOptions::setConfirmLogout (bool b)
  {
  ConfirmLogout = b;
  }
   
void UserOptions::setTimeout (int value)
  {
  timeout = value;
  }
  
int UserOptions::getTimeout (void)
  {
  return timeout;
  }
        


                       
